<?php

namespace DTInternet\DtiUsermanagement\Controller;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2022 Stefan Bublies <bublies@dt-internet.de>
 *
 *  All rights reserved
 *
 ***************************************************************/

use \TYPO3\CMS\Core\Utility\GeneralUtility;
use \TYPO3\CMS\Extbase\Mvc\Controller\ActionController;
use DTInternet\DtiUsermanagement\Utility\FeuserUtility;
use TYPO3\CMS\Extbase\Utility\LocalizationUtility as Locallang;

/**
 * (c) 2022 Stefan Bublies bublies@dt-internet.de
 */
class RegisterController extends ActionController
{
    public function registerAction()
    {
        $arguments = $this->request->getArguments();
        if (!is_null($arguments['usermanagement'])) {
            $validateFeUser = FeuserUtility::validateAndStoreFeUserData($arguments['usermanagement']);
            if (!is_null($validateFeUser) && is_array($validateFeUser)) {
                if ($validateFeUser[0]['deleted'] == 1 || $validateFeUser[0]['disable'] == 1) {
                    $usernameIsFreeDate = date('d.m.Y', strtotime("+48 hours", $validateFeUser[0]['deletedate']));
                    $this->addFlashMessage(
                        Locallang::translate('tx_dtiusermanagement.messages.useraccount.error1.body', 'dti_usermanagement', array($usernameIsFreeDate)),
                        Locallang::translate('tx_dtiusermanagement.messages.useraccount.error1.header', 'dti_usermanagement', NULL),
                        \TYPO3\CMS\Core\Messaging\AbstractMessage::ERROR
                    );
                } else {
                    $this->addFlashMessage(
                        Locallang::translate('tx_dtiusermanagement.messages.useraccount.error2.body', 'dti_usermanagement', NULL),
                        Locallang::translate('tx_dtiusermanagement.messages.useraccount.error2.header', 'dti_usermanagement', NULL),
                        \TYPO3\CMS\Core\Messaging\AbstractMessage::ERROR
                    );
                }
                $this->view->assign('registerData', $arguments['usermanagement']);
            } else {
                $this->addFlashMessage(
                    Locallang::translate('tx_dtiusermanagement.messages.useraccount.success.body', 'dti_usermanagement', NULL),
                    Locallang::translate('tx_dtiusermanagement.messages.useraccount.success.header', 'dti_usermanagement', NULL),
                    \TYPO3\CMS\Core\Messaging\AbstractMessage::OK
                );
                $this->view->assign('registerSucces', 1);
            }
        }
    }

    /**
     * (c) 2022 Stefan Bublies bublies@dt-internet.de
     */
    public function updateAction()
    {
        $arguments = $this->request->getArguments();

        $getParameters = \TYPO3\CMS\Core\Utility\GeneralUtility::_GET();
        $isItLogoutAndDelete = $getParameters['logout'];

        if($getParameters['logout'] == 1) {
            $this->view->assign('logoutAndDelete',$getParameters['logout']);
            $this->view->assign('settings', $this->settings);
        } else {
            $this->view->assign('logoutAndDelete',2);
            $this->view->assign('settings', $this->settings);
        }

        if (!is_null($arguments['usermanagement'])) {
            FeuserUtility::updateUser($arguments['usermanagement']);
            $this->addFlashMessage(
                Locallang::translate('tx_dtiusermanagement.messages.useraccount.update.success.body', 'dti_usermanagement', NULL),
                Locallang::translate('tx_dtiusermanagement.messages.useraccount.update.success.header', 'dti_usermanagement', NULL),
                \TYPO3\CMS\Core\Messaging\AbstractMessage::OK
            );
            $feUserData = FeuserUtility::get();
            $this->view->assign('registerData',$feUserData);
            $uriBuilder = $this->controllerContext->getUriBuilder();
            $uri = $uriBuilder
                ->reset()
                ->uriFor('update', array(), 'Register', 'DtiUsermanagement', 'Usermanagement');
            $this->redirectToURI($uri, 0, 301);


        } else {
            $feUserData = FeuserUtility::get();
            $this->view->assign('registerData',$feUserData);
        }

    }

    /**
     * @return void
     */
    public function deleteAction()
    {
        $arguments = $this->request->getArguments();
        $feUserData = FeuserUtility::get();
        if ($arguments['deleteuser']) {
            if ($arguments['deleteuser']['secretkey'] == $feUserData['secretkey']) {
                FeuserUtility::deleteUser($arguments['user']);
                \dti\typo3::FeUser()->logout();
                $uriBuilder = $this->controllerContext->getUriBuilder();
                $uriBuilder->setTargetPageUid($this->getSettings()['homePid']);
                $uriBuilder->setArguments(array('logout' => 1));
                $uri = $uriBuilder->build();
                $this->redirectToURI($uri, 0, 301);
            } else {
                $this->addFlashMessage(
                    Locallang::translate('tx_dtiusermanagement.user.delete.error.body', 'dti_usermanagement',
                        null),
                    Locallang::translate('tx_dtiusermanagement.user.delete.error.header',
                        'dti_usermanagement', null),
                    \TYPO3\CMS\Core\Messaging\AbstractMessage::ERROR
                );
            }
        }
    }

    /**
     * @return void
     */
    public function agentstateAction()
    {
        $arguments = $this->request->getArguments();
        $feUserData = FeuserUtility::get();
        $this->view->assignMultiple([
            'feuserData' => $feUserData,
            'formData' => $arguments['agentstate']
        ]);
        if (!is_null($arguments['agentstate'])) {
            FeuserUtility::updateState($feUserData, $arguments['agentstate']);
            $this->addFlashMessage(
                Locallang::translate('tx_dtiusermanagement.messages.agentstate.success.body', 'dti_usermanagement', NULL),
                Locallang::translate('tx_dtiusermanagement.messages.agentstate.success.header', 'dti_usermanagement', NULL),
                \TYPO3\CMS\Core\Messaging\AbstractMessage::OK
            );
            $uriBuilder = $this->controllerContext->getUriBuilder();
            $uri = $uriBuilder
                ->reset()
                ->uriFor('agentstate', array(), 'Register', 'DtiUsermanagement', 'Agentstate');
            $this->redirectToURI($uri, 0, 301);
        }
    }

    /**
     * @return void
     */
    public function passwordresetAction()
    {
        $arguments = $this->request->getArguments();
        if (!is_null($arguments['passwordreset'])) {
            $feUserData = FeuserUtility::getFeUserWithName($arguments['passwordreset']['username']);

            if ($arguments['passwordreset']['secretkey'] == $feUserData[0]['secretkey'] && !empty($feUserData)) {
                $userData = $arguments['passwordreset'];
                $userData['uid'] = $feUserData[0]['uid'];
                $resetPw = FeuserUtility::resetPasswortAnonym($userData);
                if ($resetPw == true) {
                    $this->addFlashMessage(
                        Locallang::translate('tx_dtiusermanagement.label.newpassword.success.body', 'dti_usermanagement',
                            null),
                        Locallang::translate('tx_dtiusermanagement.label.newpassword.success.header',
                            'dti_usermanagement', null),
                        \TYPO3\CMS\Core\Messaging\AbstractMessage::OK
                    );
                }
            } else {
                $this->addFlashMessage(
                    Locallang::translate('tx_dtiusermanagement.label.newpassword.error.body', 'dti_usermanagement',
                        null),
                    Locallang::translate('tx_dtiusermanagement.label.newpassword.error.header',
                        'dti_usermanagement', null),
                    \TYPO3\CMS\Core\Messaging\AbstractMessage::ERROR
                );
                $this->view->assign('formdata', $arguments['passwordreset']);
            }

        }
    }

    /**
     * Returns TypoSript settings array
     *
     * @param string $extension Name of the extension
     * @param string $plugin Name of the plugin
     * @return array
     */
    public function getSettings()
    {
        $configurationManager = GeneralUtility::makeInstance('TYPO3\\CMS\\Extbase\\Configuration\\ConfigurationManager');

        $typoScript = $configurationManager->getConfiguration(
            \TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface::CONFIGURATION_TYPE_FULL_TYPOSCRIPT);

        $settings = $typoScript['plugin.']['tx_dtiusermanagement_register.']['settings.'];

        return $settings;
    }

}
