<?php
namespace DTInternet\DtiUsermanagement\Services;

/***
 * This file is part of the "dti_usermanagement" Extension for TYPO3 CMS.
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *  (c) 2022 Stefan Bublies <bublies@dt-internet.de>, D&T Internet GmbH
 ***/

use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Core\Database\Connection;
use TYPO3\CMS\Core\Database\ConnectionPool;
use TYPO3\CMS\Core\Database\Query\QueryBuilder;

class CleanUser
{

    /**
     * * (c) 2023 Stefan Bublies
     */

    public function clean()
    {
        $this->deleteUser();
    }

    public function deleteUser()
    {

        /** @var Connection $connection */
        $connection = GeneralUtility::makeInstance(ConnectionPool::class)->getConnectionForTable('fe_users');
        /** @var QueryBuilder $queryBuilder */
        $queryBuilder = $connection->createQueryBuilder();
        $queryBuilder
            ->getRestrictions()
            ->removeAll();
        $feuserDatas = $queryBuilder
            ->select('*')
            ->from('fe_users')
            ->where($queryBuilder->expr()->eq('deleted', $queryBuilder->createNamedParameter(1, \PDO::PARAM_INT)))
            ->execute()
            ->fetchAll(\Doctrine\DBAL\FetchMode::ASSOCIATIVE);

        $time = strtotime("now");
        foreach($feuserDatas as $data) {
            $difference_in_seconds = $time - $data['deletedate'];
            if (round($difference_in_seconds/60) > 2880) {
                /** @var Connection $connection */
                $connection = GeneralUtility::makeInstance(ConnectionPool::class)->getConnectionForTable('tx_dtionlineconsulting_domain_model_topics');
                /** @var QueryBuilder $queryBuilder */
                $queryBuilder = $connection->createQueryBuilder();
                $topicDatas = $queryBuilder
                    ->select('*')
                    ->from('tx_dtionlineconsulting_domain_model_topics')
                    ->where($queryBuilder->expr()->eq('feuser', $queryBuilder->createNamedParameter((int)$data['uid'], \PDO::PARAM_INT)))
                    ->execute()
                    ->fetchAll(\Doctrine\DBAL\FetchMode::ASSOCIATIVE);


                foreach($topicDatas as $data) {
                    /** @var Connection $connection */
                    $connection = GeneralUtility::makeInstance(ConnectionPool::class)->getConnectionForTable('tx_dtionlineconsulting_domain_model_topicanswers_mm');
                    /** @var QueryBuilder $queryBuilder */
                    $queryBuilder = $connection->createQueryBuilder();
                    $topicAnswersMmDatas = $queryBuilder
                        ->select('*')
                        ->from('tx_dtionlineconsulting_domain_model_topicanswers_mm')
                        ->where($queryBuilder->expr()->eq('uid_local',
                            $queryBuilder->createNamedParameter((int)$data['uid'], \PDO::PARAM_INT)))
                        ->execute()
                        ->fetchAll(\Doctrine\DBAL\FetchMode::ASSOCIATIVE);


                    foreach($topicAnswersMmDatas as $answersMmData) {
                        $connection = GeneralUtility::makeInstance(ConnectionPool::class)->getConnectionForTable('tx_dtionlineconsulting_domain_model_topicanswers');
                        $connection->delete(
                            'tx_dtionlineconsulting_domain_model_topicanswers',
                            ['uid' => (int)$answersMmData['uid_foreign']],
                            [Connection::PARAM_INT]
                        );

                        $connection = GeneralUtility::makeInstance(ConnectionPool::class)->getConnectionForTable('tx_dtionlineconsulting_domain_model_topicanswers_mm');
                        $connection->delete(
                            'tx_dtionlineconsulting_domain_model_topicanswers_mm',
                            ['uid_local' => (int)$answersMmData['uid_local']],
                            [Connection::PARAM_INT]
                        );
                    }

                    $connection = GeneralUtility::makeInstance(ConnectionPool::class)->getConnectionForTable('tx_dtionlineconsulting_domain_model_topics');
                    $connection->delete(
                        'tx_dtionlineconsulting_domain_model_topics',
                        ['uid' => (int)$data['uid']],
                        [Connection::PARAM_INT]
                    );

                }

                $connection = GeneralUtility::makeInstance(ConnectionPool::class)->getConnectionForTable('tx_dtionlineconsulting_domain_model_videoconsulting');
                $connection->delete(
                    'tx_dtionlineconsulting_domain_model_videoconsulting',
                    ['feuser' => (int)$data['uid']],
                    [Connection::PARAM_INT]
                );


                $connection = GeneralUtility::makeInstance(ConnectionPool::class)->getConnectionForTable('fe_users');
                $connection->delete(
                    'fe_users',
                    ['uid' => (int)$data['uid']],
                    [Connection::PARAM_INT]
                );
            }
        }
    }

    /**
     * Returns TypoSript settings array
     *
     * @param string $extension Name of the extension
     * @param string $plugin Name of the plugin
     * @return array
     */
    public function getSettings()
    {
        $configurationManager = GeneralUtility::makeInstance('TYPO3\\CMS\\Extbase\\Configuration\\ConfigurationManager');

        $typoScript = $configurationManager->getConfiguration(
            \TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface::CONFIGURATION_TYPE_FULL_TYPOSCRIPT);

        $settings = $typoScript['tx_dtionlineconsulting_settings.'];

        return $settings;
    }
}
