<?php

namespace DTInternet\DtiUsermanagement\Utility;

use TYPO3\CMS\Core\SingletonInterface;
use TYPO3\CMS\Core\Utility\ArrayUtility;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Service\TypoScriptService;

/***************************************************************
 *  Copyright notice
 *  (c) 2022 Stefan Bublies <bublies@dt-internet.de>
 *  All rights reserved
 ***************************************************************/

class ArraysUtility extends \ArrayObject
{
    /**
     * 	@var mixed
     */
    protected $initialArgument;

    /**
     * 	Array konstruieren
     */
    public function __construct ( $array = null ) {
        $this->initialArgument = $array;
        if (!is_array($array)) $array = [];
        parent::__construct( $array );
        return $this;
    }

    /**
     * Ein assoziatives Array rekursiv mit einem anderen Array mergen.
     * @return array
     */
    public function merge() {
        $defaultArgs = [
            'arr1' 		=> [],
            'arr2' 		=> [],
            'addKeys' 	=> true,
            'includeEmptyValues' => false,
            'enableUnsetFeature' => true,
        ];

        $args = func_get_args();
        if ($this->initialArgument !== null && !is_array($args[1] ?? '')) {
            array_unshift($args, $this->initialArgument );
        }

        foreach ($defaultArgs as $k=>$v) {
            $val = array_shift( $args );
            if ($val == null) $val = $v;
            ${$k} = $val;
        }

        ArrayUtility::mergeRecursiveWithOverrule($arr1, $arr2, $addKeys, $includeEmptyValues, $enableUnsetFeature );
        return $arr1;
    }

    /**
     * 	Leere Werte aus einem Array entfernen.
     * 	@return array
     */
    public function removeEmpty() {
        return $this->merge( [], $this->toArray() );
    }

    /**
     * 	Einen String – oder Array – am Trennzeichen splitten, leere Elemente entfernen
     * 	Funktioniert mit Strings und Arrays.
     * 	@return array
     */
    public function trimExplode( $delimiter = ',', $removeEmpty = true ) {
        $arr = $this->initialArgument !== null ? $this->initialArgument : (array) $this;

        if ($delimiter === false || $delimiter === true) {
            $delimiter = ',';
            $removeEmpty = $delimiter;
        }

        $firstDelimiter = is_array($delimiter) ? $delimiter[0] : $delimiter;

        if (is_array($arr)) $arr = join($firstDelimiter, $arr);
        if (is_array($delimiter)) {
            foreach ($delimiter as $d) {
                $arr = str_replace( $d, $firstDelimiter, $arr);
            }
            $delimiter = $firstDelimiter;
        }
        $arr = GeneralUtility::trimExplode( $delimiter, $arr, $removeEmpty );
        return $arr;
    }

    /**
     * 	Einen String – oder Array – am Trennzeichen splitten, nicht numerische
     * 	und leere Elemente entfernen
     * 	@return array
     */
    public function intExplode( $delimiter = ',' ) {
        $finals = [];
        if ($arr = $this->trimExplode($delimiter)) {
            foreach ($arr as $k=>$v) {
                if (is_numeric($v)) $finals[] = $v;
            }
        }
        return $finals;
    }

    /**
     * 	Als Key des Arrays ein Feld im Array verwenden, z.B. um eine Liste zu bekommen,
     * 	deren Key immer die UID des assoziativen Arrays ist:
     * 	@return array
     */
    public function key( $key = 'uid', $value = false ) {

        $arr = (array) $this;
        $values = $value === false ? array_values($arr) : array_column( $arr, $value );
        $combinedArray = array_combine( array_column($arr, $key), $values );

        $this->exchangeArray($combinedArray);
        return $this;
    }

    /**
     * 	Assoziatives Array auf bestimmte Elemente reduzieren / destillieren:

     * 	@return array
     */
    public function pluck( $keys = null, $isSingleObject = false ) {

        $arr = (array) $this;

        $pluckedArray = [];
        if ($getSingleKey = is_string($keys)) {
            $keys = [$keys];
        }

        if ($isSingleObject) {
            $arr = [$arr];
        }

        foreach ($keys as $key) {
            foreach ($arr as $n=>$v) {
                if ($getSingleKey) {
                    $pluckedArray[$n] = $v[$key] ?? '';
                } else {
                    if (!isset($pluckedArray[$n])) $pluckedArray[$n] = [];
                    $pluckedArray[$n][$key] = $v[$key] ?? '';
                }
            }
        }

        if ($isSingleObject) {
            $pluckedArray = array_pop($pluckedArray);
        }

        $this->exchangeArray($pluckedArray);
        return $this;
    }


    /**
     * 	Gibt dieses Array-Object als "normales" Array zurück.
     * 	@return array
     */
    public function toArray () {
        return (array) $this;
    }

    /**
     * 	Gibt das erste Element des Arrays zurück, ohne array_shift()
     * 	@return array
     */
    public function first () {
        $arr = (array) $this;
        if (!$arr) return false;
        foreach ($arr as $k=>$v) return $v;
    }
}
